<?php
/**
 * User: nester_a
 * Date: 25.06.14
 */

namespace LMS\Libraries\Database;

class Driver extends \JDatabaseDriver implements Interfaces\IDriver
{
	/**
	 * @var \JDatabaseDriver
	 */
	private $decoratedDriver;

	public $name;

	public function __construct(\JDatabaseDriver $driver)
	{
		$this->decoratedDriver = $driver;
		$this->name = $driver->name;
	}

	/**
	 * Test to see if the connector is available.
	 *
	 * @return  boolean  True on success, false otherwise.
	 *
	 * @since   11.2
	 */
	public static function isSupported()
	{
		return true;
	}

	/**
	 * Connects to the database if needed.
	 *
	 * @return  void  Returns void if the database connected successfully.
	 *
	 * @since   12.1
	 * @throws  \RuntimeException
	 */
	public function connect()
	{
		$this->decoratedDriver->connect();
	}

	/**
	 * Determines if the connection to the server is active.
	 *
	 * @return  boolean  True if connected to the database engine.
	 *
	 * @since   11.1
	 */
	public function connected()
	{
		return $this->decoratedDriver->connected();
	}

	/**
	 * Disconnects the database.
	 *
	 * @return  void
	 *
	 * @since   12.1
	 */
	public function disconnect()
	{
		$this->decoratedDriver->disconnect();
	}

	/**
	 * Drops a table from the database.
	 *
	 * @param   string $table     The name of the database table to drop.
	 * @param   boolean $ifExists  Optionally specify that the table must exist before it is dropped.
	 *
	 * @return  \JDatabaseDriver     Returns this object to support chaining.
	 *
	 * @since   11.4
	 * @throws  \RuntimeException
	 */
	public function dropTable($table, $ifExists = true)
	{
		return $this->decoratedDriver->dropTable($table, $ifExists);
	}

	/**
	 * Method to escape a string for usage in an SQL statement.
	 *
	 * @param   string $text   The string to be escaped.
	 * @param   boolean $extra  Optional parameter to provide extra escaping.
	 *
	 * @return  string   The escaped string.
	 *
	 * @since   11.1
	 */
	public function escape($text, $extra = false)
	{
		return $this->decoratedDriver->escape($text, $extra);
	}

	/**
	 * Method to fetch a row from the result set cursor as an array.
	 *
	 * @param   mixed $cursor  The optional result set cursor from which to fetch the row.
	 *
	 * @return  mixed  Either the next row from the result set or false if there are no more rows.
	 *
	 * @since   11.1
	 */
	protected function fetchArray($cursor = null)
	{
		return $this->decoratedDriver->fetchArray($cursor);
	}

	/**
	 * Method to fetch a row from the result set cursor as an associative array.
	 *
	 * @param   mixed $cursor  The optional result set cursor from which to fetch the row.
	 *
	 * @return  mixed  Either the next row from the result set or false if there are no more rows.
	 *
	 * @since   11.1
	 */
	protected function fetchAssoc($cursor = null)
	{
		return $this->decoratedDriver->fetchAssoc($cursor);
	}

	/**
	 * Method to fetch a row from the result set cursor as an object.
	 *
	 * @param   mixed $cursor  The optional result set cursor from which to fetch the row.
	 * @param   string $class   The class name to use for the returned row object.
	 *
	 * @return  mixed   Either the next row from the result set or false if there are no more rows.
	 *
	 * @since   11.1
	 */
	protected function fetchObject($cursor = null, $class = 'stdClass')
	{
		if($class != 'stdClass'){
			return $this->decoratedDriver->fetchObject($cursor, $class);
		}

		$object = $this->decoratedDriver->fetchObject($cursor, '\\LMS\\Libraries\Database\\Object');
		$query = $this->getQuery();
		if(is_string($query)){
			return $object;
		}
		if(!is_null($object)){
			if(method_exists($object, 'setTable')){
				$object->setTable($this->getQuery()->getTable());
			}
		}
		return $object;
	}

	/**
	 * Method to free up the memory used for the result set.
	 *
	 * @param   mixed $cursor  The optional result set cursor from which to fetch the row.
	 *
	 * @return  void
	 *
	 * @since   11.1
	 */
	protected function freeResult($cursor = null)
	{
		$this->decoratedDriver->freeResult($cursor);
	}

	/**
	 * Get the number of affected rows for the previous executed SQL statement.
	 *
	 * @return  integer  The number of affected rows.
	 *
	 * @since   11.1
	 */
	public function getAffectedRows()
	{
		return $this->decoratedDriver->getAffectedRows();
	}

	/**
	 * Method to get the database collation in use by sampling a text field of a table in the database.
	 *
	 * @return  mixed  The collation in use by the database or boolean false if not supported.
	 *
	 * @since   11.1
	 */
	public function getCollation()
	{
		return $this->decoratedDriver->getCollation();
	}

	/**
	 * Get the number of returned rows for the previous executed SQL statement.
	 *
	 * @param   resource $cursor  An optional database cursor resource to extract the row count from.
	 *
	 * @return  integer   The number of returned rows.
	 *
	 * @since   11.1
	 */
	public function getNumRows($cursor = null)
	{
		return $this->decoratedDriver->getNumRows($cursor);
	}

	/**
	 * Retrieves field information about the given tables.
	 *
	 * @param   string $table     The name of the database table.
	 * @param   boolean $typeOnly  True (default) to only return field types.
	 *
	 * @return  array  An array of fields by table.
	 *
	 * @since   11.1
	 * @throws  \RuntimeException
	 */
	public function getTableColumns($table, $typeOnly = true)
	{
		return $this->decoratedDriver->getTableColumns($table, $typeOnly);
	}

	/**
	 * Shows the table CREATE statement that creates the given tables.
	 *
	 * @param   mixed $tables  A table name or a list of table names.
	 *
	 * @return  array  A list of the create SQL for the tables.
	 *
	 * @since   11.1
	 * @throws  \RuntimeException
	 */
	public function getTableCreate($tables)
	{
		return $this->decoratedDriver->getTableCreate($tables);
	}

	/**
	 * Retrieves field information about the given tables.
	 *
	 * @param   mixed $tables  A table name or a list of table names.
	 *
	 * @return  array  An array of keys for the table(s).
	 *
	 * @since   11.1
	 * @throws  \RuntimeException
	 */
	public function getTableKeys($tables)
	{
		return $this->decoratedDriver->getTableKeys($tables);
	}

	/**
	 * Method to get an array of all tables in the database.
	 *
	 * @return  array  An array of all the tables in the database.
	 *
	 * @since   11.1
	 * @throws  \RuntimeException
	 */
	public function getTableList()
	{
		return $this->decoratedDriver->getTableList();
	}

	/**
	 * Get the version of the database connector
	 *
	 * @return  string  The database connector version.
	 *
	 * @since   11.1
	 */
	public function getVersion()
	{
		return $this->decoratedDriver->getVersion();
	}

	/**
	 * Method to get the auto-incremented value from the last INSERT statement.
	 *
	 * @return  integer  The value of the auto-increment field from the last inserted row.
	 *
	 * @since   11.1
	 */
	public function insertid()
	{
		return $this->decoratedDriver->insertid();
	}

	/**
	 * Locks a table in the database.
	 *
	 * @param   string $tableName  The name of the table to unlock.
	 *
	 * @return  \JDatabaseDriver     Returns this object to support chaining.
	 *
	 * @since   11.4
	 * @throws  \RuntimeException
	 */
	public function lockTable($tableName)
	{
		return $this->decoratedDriver->lockTable($tableName);
	}

	/**
	 * Renames a table in the database.
	 *
	 * @param   string $oldTable  The name of the table to be renamed
	 * @param   string $newTable  The new name for the table.
	 * @param   string $backup    Table prefix
	 * @param   string $prefix    For the table - used to rename constraints in non-mysql databases
	 *
	 * @return  \JDatabaseDriver    Returns this object to support chaining.
	 *
	 * @since   11.4
	 * @throws  \RuntimeException
	 */
	public function renameTable($oldTable, $newTable, $backup = null, $prefix = null)
	{
		return $this->decoratedDriver->renameTable($oldTable, $newTable, $backup, $prefix);
	}

	/**
	 * Select a database for use.
	 *
	 * @param   string $database  The name of the database to select for use.
	 *
	 * @return  boolean  True if the database was successfully selected.
	 *
	 * @since   11.1
	 * @throws  \RuntimeException
	 */
	public function select($database)
	{
		return $this->decoratedDriver->select($database);
	}

	/**
	 * Set the connection to use UTF-8 character encoding.
	 *
	 * @return  boolean  True on success.
	 *
	 * @since   11.1
	 */
	public function setUTF()
	{
		return $this->decoratedDriver->setUTF();
	}

	/**
	 * Method to commit a transaction.
	 *
	 * @return  void
	 *
	 * @since   11.1
	 * @throws  \RuntimeException
	 */
	public function transactionCommit()
	{
		$this->decoratedDriver->transactionCommit();
	}

	/**
	 * Method to roll back a transaction.
	 *
	 * @return  void
	 *
	 * @since   11.1
	 * @throws  \RuntimeException
	 */
	public function transactionRollback()
	{
		$this->decoratedDriver->transactionRollback();
	}

	/**
	 * Method to initialize a transaction.
	 *
	 * @return  void
	 *
	 * @since   11.1
	 * @throws  \RuntimeException
	 */
	public function transactionStart()
	{
		$this->decoratedDriver->transactionStart();
	}

	/**
	 * Execute the SQL statement.
	 *
	 * @return  mixed  A database cursor resource on success, boolean false on failure.
	 *
	 * @since   12.1
	 * @throws  \RuntimeException
	 */
	public function execute()
	{
		return $this->decoratedDriver->execute();
	}

	/**
	 * Unlocks tables in the database.
	 *
	 * @return  \JDatabaseDriver  Returns this object to support chaining.
	 *
	 * @since   11.4
	 * @throws  \RuntimeException
	 */
	public function unlockTables()
	{
		return $this->decoratedDriver->unlockTables();
	}


	public function getPrefix()
	{
		return $this->decoratedDriver->getPrefix();
	}

	public function getQuery($new = false)
	{
		if($new){
			return new Query($this->decoratedDriver->getQuery($new), $this);
		}else{
			return $this->decoratedDriver->getQuery($new);
		}
	}

	/**
	 * @param \JDatabaseQuery|string $query
	 * @param int $offset
	 * @param int $limit
	 * @return \JDatabaseDriver
	 */
	public function setQuery($query, $offset = 0, $limit = 0)
	{
		return $this->decoratedDriver->setQuery($query, $offset, $limit);
	}

	public function setLimit($offset = 0, $limit = 0)
	{
		$this->decoratedDriver->limit = $limit;
		$this->decoratedDriver->offset = $offset;
	}


}